"use strict";
exports.__esModule = true;
exports.isSchema = exports.isOAS31 = exports.isRef = void 0;
/**
 * @param check Data to determine if it contains a ReferenceObject (`$ref` pointer`).
 * @returns If the supplied data has a `$ref` pointer.
 */
function isRef(check) {
    return check.$ref !== undefined;
}
exports.isRef = isRef;
/**
 * @param check API definition to determine if it's a 3.1 definition.
 * @returns If the definition is a 3.1 definition.
 */
function isOAS31(check) {
    return check.openapi === '3.1.0';
}
exports.isOAS31 = isOAS31;
/**
 * @param check JSON Schema object to determine if it's a non-polymorphic schema.
 * @param isPolymorphicAllOfChild If this JSON Schema object is the child of a polymorphic `allOf`.
 * @returns If the JSON Schema object is a JSON Schema object.
 */
function isSchema(check, isPolymorphicAllOfChild) {
    if (isPolymorphicAllOfChild === void 0) { isPolymorphicAllOfChild = false; }
    return (check.type !== undefined ||
        check.allOf !== undefined ||
        check.anyOf !== undefined ||
        check.oneOf !== undefined ||
        isPolymorphicAllOfChild);
}
exports.isSchema = isSchema;
