
const fs = require('fs')
const { DownloaderHelper } = require('node-downloader-helper');
const util = require('util');
const exec = util.promisify(require('child_process').exec);
const { s3Upload } = require('./aws-lib')
const axios = require('axios').default;
const { v4: uuidv4 } = require('uuid');


const getFrame = async (url, camp_id, duration, upload_dir_path) => {

    try {

        return new Promise((resolve, reject) => {
            const file_name = `${camp_id}.mp4`

            const dl = new DownloaderHelper(url, 'temp', {
                method: 'GET', // Request Method Verb
                headers: {},  // Custom HTTP Header ex: Authorization, User-Agent
                fileName: file_name,// Custom filename when saved
                retry: false, // { maxRetries: number, delay: number in ms } or false to disable (default)
                forceResume: false, // If the server does not return the "accept-ranges" header, can be force if it does support it
                removeOnStop: true, // remove the file when is stopped (default:true)
                removeOnFail: true, // remove the file when fail (default:true)
                override: true,
                httpRequestOptions: {}, // Override the http request options  
                httpsRequestOptions: {}, // Override t
            });

            dl.on('end', async (response) => {
                const ffmpeg = process.env.vidamaze_FFMPEG_PATH
                const file_name = `${camp_id}.jpg`
                const output_path = process.cwd() + `/temp/${file_name}`;
                if (fs.existsSync(output_path)) {
                    fs.unlinkSync(output_path)
                }


                await exec(`${ffmpeg} -ss ${duration} -i temp/${response.fileName} -frames:v 1 -q:v 2 ${output_path} 2>&1`)
                // fs.unlinkSync(`temp/${response.fileName}`)

                const url = await s3Upload(`${process.cwd()}/temp/`, file_name, upload_dir_path)

                console.log(url)
                if (!url.url) resolve({ error: 'Something went wrong' });

                fs.unlinkSync(`temp/${file_name}`)
                console.log('Video Downloaded and Frame Created', url)
                resolve(url)


            })
            dl.start();

            console.log("Hello")
        })

    } catch (error) {
        console.log(error)
    }

}

const createGifFromVideo = async (url, camp_id, upload_dir_path,start,end,gif_url) => {

    return new Promise(async (resolve, reject) => {
        try {
            const ffmpeg = process.env.vidamaze_FFMPEG_PATH
            const download_file_name = `${camp_id}.mp4`
            // console.log("gifduration="+start,end);
            if (!fs.existsSync(download_file_name)) {
                const dl = new DownloaderHelper(url, 'temp', {
                    method: 'GET', // Request Method Verb
                    headers: {},  // Custom HTTP Header ex: Authorization, User-Agent
                    fileName: download_file_name,// Custom filename when saved
                    retry: false, // { maxRetries: number, delay: number in ms } or false to disable (default)
                    forceResume: false, // If the server does not return the "accept-ranges" header, can be force if it does support it
                    removeOnStop: true, // remove the file when is stopped (default:true)
                    removeOnFail: true, // remove the file when fail (default:true)
                    override: true,
                    httpRequestOptions: {}, // Override the http request options  
                    httpsRequestOptions: {}, // Override t
                });

                dl.on('end', async (response) => {

                    const file_name = `${uuidv4()}.gif`
                    const output_path = process.cwd() + `/temp/${file_name}`;

                    if (fs.existsSync(output_path)) {
                        fs.unlinkSync(output_path)
                    }

                    // await exec(`${ffmpeg} -i temp/${response.fileName} -vf "scale=640:-1:flags=lanczos,split[s0][s1];[s0]palettegen[p];[s1][p]paletteuse" -loop 0 ${output_path} 2>&1`)
                    //ffmpeg -ss 30 -t 3 -i input.mp4 -vf "fps=10,scale=320:-1:flags=lanczos,split[s0][s1];[s0]palettegen[p];[s1][p]paletteuse" -loop 0 output.gif
                    if(start && end){
                        // console.log('here')
                        // console.log(`${ffmpeg} -ss ${start} -t ${end-start} -i temp/${response.fileName} -vf "fps=10,scale=480:-1:flags=lanczos,split[s0][s1];[s0]palettegen[p];[s1][p]paletteuse" -loop 0 ${output_path} 2>&1`)
                        await exec(`${ffmpeg} -ss ${start} -t ${end-start} -i temp/${response.fileName} -vf "fps=10,scale=480:-1:flags=lanczos,split[s0][s1];[s0]palettegen[p];[s1][p]paletteuse" -loop 0 ${output_path} 2>&1`)
                    }else{
                        await exec(`${ffmpeg} -i temp/${response.fileName} -vf "fps=30,scale=320:-1,split [a][b];[a] palettegen [p];[b][p] paletteuse" ${output_path} 2>&1`)
                    }
                    
                    fs.unlinkSync(`temp/${response.fileName}`)

                    const gif = await s3Upload(`${process.cwd()}/temp/`, file_name, upload_dir_path,gif_url)
                    if (!gif.url) 
                    {
                        resolve({ error: 'Something went wrong' });
                    }else{
                        // console.log(gif.url)
                        fs.unlinkSync(`temp/${file_name}`)
                        const url = gif.url.replace("d37q3r06begyqi.cloudfront.net","engageleads-gallery.s3.us-west-2.amazonaws.com");
                        // console.log(url)
                        resolve(url)
                    }
                })
                dl.on('progress',async (info) => {
                    console.log(info)
                })
                dl.start().catch(err => {
                    console.error(err)
                    reject(error)
                });
            } else {

                const file_name = `${uuidv4()}.gif`
                const output_path = process.cwd() + `/temp/${file_name}`;

                if (fs.existsSync(output_path)) {
                    fs.unlinkSync(output_path)
                }

                // await exec(`${ffmpeg} -i temp/${response.fileName} -vf "scale=640:-1:flags=lanczos,split[s0][s1];[s0]palettegen[p];[s1][p]paletteuse" -loop 0 ${output_path} 2>&1`)
                // console.log(`${ffmpeg} -i temp/${download_file_name} -vf "fps=10,scale=480:-1,split [a][b];[a] palettegen [p];[b][p] paletteuse" ${output_path} 2>&1`)
                if(start && end){
                    // console.log('here else')
                    await exec(`${ffmpeg} -ss ${start} -t ${end-start} -i temp/${download_file_name} -vf "fps=10,scale=480:-1:flags=lanczos,split[s0][s1];[s0]palettegen[p];[s1][p]paletteuse" -loop 0 ${output_path} 2>&1`)
                }else{
                    await exec(`${ffmpeg} -i temp/${download_file_name} -vf "fps=30,scale=320:-1,split [a][b];[a] palettegen [p];[b][p] paletteuse" ${output_path} 2>&1`)
                }
                fs.unlinkSync(`temp/${download_file_name}`)

                const gif = await s3Upload(`${process.cwd()}/temp/`, file_name, upload_dir_path,gif_url)

                if (!gif.url) {
                    resolve({ error: 'Something went wrong' });
                }else{
                     // console.log(url.url)
                    fs.unlinkSync(`temp/${file_name}`)
                    const url = gif.url.replace("d37q3r06begyqi.cloudfront.net","engageleads-gallery.s3.us-west-2.amazonaws.com");

                    resolve(url)
                }

            }
        } catch (error) {
            reject(error)
        }

    })
}


trimVideo = (url, start_time, end_time, upload_dir_path) => {


    return new Promise(async (resolve, reject) => {
        try {
            // const ffmpeg = process.env.vidamaze_FFMPEG_PATH
            const ffmpegPath = require('@ffmpeg-installer/ffmpeg').path
            const ffmpeg = require('fluent-ffmpeg')
            ffmpeg.setFfmpegPath(ffmpegPath)
            const download_file_name = `${(Math.random() + 1).toString(36).substring(2)}.mp4`
            const start = convertHMS(start_time);
            const end = convertHMS(end_time)

            // console.log(ffmpegPath,ffmpeg)
            
            if(url.includes("&profile_id") || url.includes("oauth2_token_id")){
                const output_path = process.cwd() + `/temp/${download_file_name}`;
                var file = fs.createWriteStream(output_path);


                const response = await axios({
                    method: 'GET',
                    url: url,
                    responseType: 'stream',
                  });
              
                  const w = response.data.pipe(file);
                  w.on('finish', async () => {
                    // console.log('Successfully downloaded file!',w,output_path);
                    const file_name = `${(Math.random() + 11).toString(36).substring(3)}.mp4`
                    const trim_path = process.cwd() + `/temp/${file_name}`;
    
                    if (fs.existsSync(trim_path)) {
                        fs.unlinkSync(trim_path)
                    }
                    // console.log(response.fileName)
                    // await exec(`${ffmpeg} -i temp/${response.fileName} -vf "scale=640:-1:flags=lanczos,split[s0][s1];[s0]palettegen[p];[s1][p]paletteuse" -loop 0 ${output_path} 2>&1`)
                    // await exec(`${ffmpeg} -i temp/${download_file_name} -ss ${start} -to ${end} ${trim_path} -c copy 2>&1`)
                    ffmpeg(`temp/${download_file_name}`)
                        .setStartTime(start)
                        .setDuration(end)
                        .output(trim_path)
                        .on('end', async function(err) {
                            if(!err) { console.log('conversion Done') }
                            
                            fs.unlinkSync(`temp/${download_file_name}`)
    
                            const url = await s3Upload(`${process.cwd()}/temp/`, file_name, upload_dir_path)
    
    
                            if (!url.url) resolve({ error: 'Something went wrong' });
    
                            // console.log(url.url)
                            fs.unlinkSync(`temp/${file_name}`)
    
                            resolve(url)
                        })
                        .on('progress', async function(progress) {
                            console.log('Processing: ' + progress + '% done');
                        })
                        .on('error', function(err){
                            console.log('error: ', err)
                        }).run()
                    // console.log(`${ffmpeg} -i temp/${response.fileName} -ss ${start} -to ${end} ${output_path} -async 1 2>&1`)
                    
                  });
            }else{
                const dl = new DownloaderHelper(url, 'temp', {
                    method: 'GET', // Request Method Verb
                    headers: {},  // Custom HTTP Header ex: Authorization, User-Agent
                    fileName: download_file_name,// Custom filename when saved
                    retry: false, // { maxRetries: number, delay: number in ms } or false to disable (default)
                    forceResume: false, // If the server does not return the "accept-ranges" header, can be force if it does support it
                    removeOnStop: true, // remove the file when is stopped (default:true)
                    removeOnFail: true, // remove the file when fail (default:true)
                    override: true,
                    httpRequestOptions: {}, // Override the http request options  
                    httpsRequestOptions: {}, // Override t
                });
    
                dl.on('end', async (response) => {
    
                    const file_name = `${(Math.random() + 1).toString(36).substring(2)}.mp4`
                    const output_path = process.cwd() + `/temp/${file_name}`;
    
                    if (fs.existsSync(output_path)) {
                        fs.unlinkSync(output_path)
                    }
    
                    // await exec(`${ffmpeg} -i temp/${response.fileName} -vf "scale=640:-1:flags=lanczos,split[s0][s1];[s0]palettegen[p];[s1][p]paletteuse" -loop 0 ${output_path} 2>&1`)
                    // await exec(`${ffmpeg} -i temp/${response.fileName} -ss ${start} -to ${end} ${output_path} -c copy 2>&1`)
                    // console.log(response)
                    ffmpeg(`temp/${response.fileName}`)
                    .setStartTime(start)
                    .setDuration(end)
                    .output(output_path)
                    .on('end', async function(err) {
                        if(!err) { console.log('conversion Done') }
                        
                        fs.unlinkSync(`temp/${response.fileName}`)
    
                        const url = await s3Upload(`${process.cwd()}/temp/`, file_name, upload_dir_path)
    
    
                        if (!url.url) resolve({ error: 'Something went wrong' });
    
                        // console.log(url.url)
                        fs.unlinkSync(`temp/${file_name}`)
    
                        resolve(url)
                    })
                    .on('progress', async function(progress) {
                        console.log('Processing: ' + progress.percent + '% done');
                    })
                    .on('error', function(err){
                        console.log('error: ', err)
                    }).run()
                    // console.log(`${ffmpeg} -i temp/${response.fileName} -ss ${start} -to ${end} ${output_path} -async 1 2>&1`)
                    
                    // console.log('Download Completed', response)
                })
                dl.start();
            }
            
        } catch (error) {
            console.log(error)
        }
    })
}


convertHMS = (value) => {
    const sec = parseInt(value, 10); // convert value to number if it's string
    let hours = Math.floor(sec / 3600); // get hours
    let minutes = Math.floor((sec - (hours * 3600)) / 60); // get minutes
    let seconds = sec - (hours * 3600) - (minutes * 60); //  get seconds
    // add 0 if value < 10; Example: 2 => 02
    if (hours < 10) { hours = "0" + hours; }
    if (minutes < 10) { minutes = "0" + minutes; }
    if (seconds < 10) { seconds = "0" + seconds; }
    return hours + ':' + minutes + ':' + seconds; // Return is HH : MM : SS
}


module.exports = {
    getFrame,
    createGifFromVideo,
    trimVideo
}